@@ -67,16 +67,11 @@ typedef struct _ShiftSlot {
     float scale;           /* size scale (fit to maximal thumb size */
     float opacity;
     float rotation;
-    
-    GLfloat opacityVelocity;
 
     GLfloat tx;
     GLfloat ty;
-    GLfloat topacity;
 
-
-    Bool    adjust;
-    Bool    active;
+    Bool    primary;
 
 } ShiftSlot;
 
@@ -101,6 +96,7 @@ typedef struct _ShiftScreen {
 
     PreparePaintScreenProc preparePaintScreen;
     DonePaintScreenProc    donePaintScreen;
+    PaintScreenProc        paintScreen;
     PaintOutputProc        paintOutput;
     PaintWindowProc        paintWindow;
     DamageWindowRectProc   damageWindowRect;
@@ -145,9 +141,11 @@ typedef struct _ShiftScreen {
     CompOutput *output;
     int        usedOutput;
 
-    float reflect;
-    float reflectVelocity;
+    float anim;
+    float animVelocity;
+    
     float reflectBrightness;
+    Bool  reflectActive;
 
     int	  buttonPressTime;
     Bool  buttonPressed;
@@ -155,6 +153,8 @@ typedef struct _ShiftScreen {
     int   startY;
     float startTarget;
     float lastTitle;
+
+    Bool  paintingAbove;
 } ShiftScreen;
 
 typedef struct _ShiftWindow {
@@ -164,8 +164,8 @@ typedef struct _ShiftWindow {
     float brightness;
     float opacityVelocity;
     float brightnessVelocity;
-    
-    Bool      active;
+
+    Bool active;
 } ShiftWindow;
 
 #define PI 3.1415926
@@ -292,7 +292,15 @@ shiftRenderWindowTitle (CompScreen *s)
 	return;
 
     int ox1, ox2, oy1, oy2;
-    getCurrentOutputExtents (s, &ox1, &oy1, &ox2, &oy2);
+
+    if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
+    {
+	ox1 = oy1 = 0;
+	ox2 = s->width;
+	oy2 = s->height;
+    }
+    else
+	getCurrentOutputExtents (s, &ox1, &oy1, &ox2, &oy2);
 
     /* 75% of the output device as maximum width */
     tA.maxwidth = (ox2 - ox1) * 3 / 4;
@@ -340,10 +348,21 @@ shiftDrawWindowTitle (CompScreen *s)
     float height = ss->textHeight;
     float border = 10.0f;
 
-    int ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
-    int ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
-    int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
-    int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    int ox1, ox2, oy1, oy2;
+
+    if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
+    {
+	ox1 = oy1 = 0;
+	ox2 = s->width;
+	oy2 = s->height;
+    }
+    else
+    {
+        ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
+        ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
+        oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
+        oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    }
 
     float x = ox1 + ((ox2 - ox1) / 2) - (ss->textWidth / 2);
     float y;
@@ -473,14 +492,13 @@ shiftPaintWindow (CompWindow		 *w,
     Bool       status;
 
     SHIFT_SCREEN (s);
+    SHIFT_WINDOW (w);
 
-    if (ss->state != ShiftStateNone)
+    if (ss->state != ShiftStateNone && !ss->paintingAbove)
     {
 	WindowPaintAttrib sAttrib = *attrib;
 	Bool		  scaled = FALSE;
 
-	SHIFT_WINDOW (w);
-
     	if (w->mapNum)
 	{
 	    if (!w->texture->pixmap && !w->bindFailed)
@@ -488,7 +506,7 @@ shiftPaintWindow (CompWindow		 *w,
 	}
 
 	
-	if (sw->active || sw->slots[0].adjust || sw->slots[1].adjust)
+	if (sw->active)
 	    scaled = (ss->activeSlot != NULL);
 	
 	if (sw->opacity > 0.0 && (ss->activeSlot == NULL))
@@ -499,6 +517,11 @@ shiftPaintWindow (CompWindow		 *w,
 	else
 	    mask |= PAINT_WINDOW_NO_CORE_INSTANCE_MASK;
 
+	if (sw->active &&
+	    (ss->output->id == ss->usedOutput || ss->output->id == ~0))
+	    mask |= PAINT_WINDOW_NO_CORE_INSTANCE_MASK;
+	
+
 	UNWRAP (ss, s, paintWindow);
 	status = (*s->paintWindow) (w, &sAttrib, transform, region, mask);
 	WRAP (ss, s, paintWindow, shiftPaintWindow);
@@ -509,34 +532,50 @@ shiftPaintWindow (CompWindow		 *w,
 	    CompTransform  wTransform = *transform;
 	    ShiftSlot      *slot = ss->activeSlot->slot;
 
+	    float sx     = ss->anim * slot->tx;
+	    float sy     = ss->anim * slot->ty;
+	    float sz     = ss->anim * slot->z;
+	    float sscale = (ss->anim * slot->scale) + (1 - ss->anim);
+	    float srot   = (ss->anim * slot->rotation);
+
+	    float sopacity;
+	    
+	    if (slot->primary && !ss->reflectActive)
+		sopacity = (ss->anim * slot->opacity) + (1 - ss->anim);
+	    else
+		sopacity = ss->anim * slot->opacity;
+
+	    if (sopacity <= 0.0)
+		return status;
+
 	    if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK)
 		return FALSE;
 
-	    initFragmentAttrib (&fragment, &w->paint);
+	    initFragmentAttrib (&fragment, attrib);
 
-	    fragment.opacity    = (float)fragment.opacity * slot->topacity;
+	    fragment.opacity    = (float)fragment.opacity * sopacity;
 	    fragment.brightness = (float)fragment.brightness *
 				  ss->reflectBrightness;
 
 	    if (w->alpha || fragment.opacity != OPAQUE)
 		mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
 
-	    matrixTranslate (&wTransform, slot->tx, slot->ty, slot->z);
+	    matrixTranslate (&wTransform, sx, sy, sz);
 
 	    matrixTranslate (&wTransform,
-			     w->attrib.x + (w->width  * slot->scale / 2),
-			     w->attrib.y + (w->height  * slot->scale / 2.0),
+			     w->attrib.x + (w->width  * sscale / 2),
+			     w->attrib.y + (w->height  * sscale / 2.0),
 			     0.0f);
 
 	    matrixScale (&wTransform, ss->output->width, -ss->output->height,
                 	 1.0f);
 	
-	    matrixRotate (&wTransform, slot->rotation, 0.0, 1.0, 0.0);
+	    matrixRotate (&wTransform, srot, 0.0, 1.0, 0.0);
 
 	    matrixScale (&wTransform, 1.0f  / ss->output->width,
                 	 -1.0f / ss->output->height, 1.0f);
 
-	    matrixScale (&wTransform, slot->scale, slot->scale, 1.0f);
+	    matrixScale (&wTransform, sscale, sscale, 1.0f);
 	    matrixTranslate (&wTransform, -w->attrib.x - (w->width / 2),
 			     -w->attrib.y - (w->height / 2), 0.0f);
 
@@ -569,8 +608,15 @@ shiftPaintWindow (CompWindow		 *w,
 		ShiftOverlayIconEnum iconOverlay = shiftGetOverlayIcon (s);
 		ShiftSlot      *slot = ss->activeSlot->slot;
 
-		scaledWinWidth  = w->width  * slot->scale;
-		scaledWinHeight = w->height * slot->scale;
+		float sx       = ss->anim * slot->tx;
+		float sy       = ss->anim * slot->ty;
+		float sz       = ss->anim * slot->z;
+		float sscale   = (ss->anim * slot->scale) + (1 - ss->anim);
+		float srot     = (ss->anim * slot->rotation);
+		float sopacity = ss->anim * slot->opacity;
+
+		scaledWinWidth  = w->width  * sscale;
+		scaledWinHeight = w->height * sscale;
 
 		if (!w->texture->pixmap)
 		    iconOverlay = OverlayIconBig;
@@ -640,30 +686,30 @@ shiftPaintWindow (CompWindow		 *w,
 			sAttrib.opacity = w->opacity;
 		    }
 
-		    initFragmentAttrib (&fragment, &sAttrib);
+		    initFragmentAttrib (&fragment, attrib);
 
-		    fragment.opacity = (float)fragment.opacity * slot->topacity;
+		    fragment.opacity = (float)fragment.opacity * sopacity;
 		    fragment.brightness = (float)fragment.brightness *
 					  ss->reflectBrightness;
 
-		    matrixTranslate (&wTransform, slot->tx, slot->ty, slot->z);
+		    matrixTranslate (&wTransform, sx, sy, sz);
 
 		    matrixTranslate (&wTransform, w->attrib.x +
-				     (w->width  * slot->scale / 2),
+				     (w->width  * sscale / 2),
 				     w->attrib.y +
-				     (w->height  * slot->scale / 2.0), 0.0f);
+				     (w->height  * sscale / 2.0), 0.0f);
 	
 		    matrixScale (&wTransform, ss->output->width,
                 		 -ss->output->height, 1.0f);
 
-		    matrixRotate (&wTransform, slot->rotation, 0.0, 1.0, 0.0);
+		    matrixRotate (&wTransform, srot, 0.0, 1.0, 0.0);
 
 		    matrixScale (&wTransform, 1.0f  / ss->output->width,
                 		 -1.0f / ss->output->height, 1.0f);
 
 		    matrixTranslate (&wTransform, x -
-				     (w->width  * slot->scale / 2), y -
-				     (w->height  * slot->scale / 2.0), 0.0f);
+				     (w->width  * sscale / 2), y -
+				     (w->height  * sscale / 2.0), 0.0f);
 		    matrixScale (&wTransform, scale, scale, 1.0f);
 
 		    glPushMatrix ();
@@ -681,30 +727,53 @@ shiftPaintWindow (CompWindow		 *w,
     }
     else
     {
+	WindowPaintAttrib sAttrib = *attrib;
+	
+	if (ss->paintingAbove)
+	{
+	    	sAttrib.opacity = sAttrib.opacity * (1.0 - ss->anim);
+	}
+	
 	UNWRAP (ss, s, paintWindow);
-	status = (*s->paintWindow) (w, attrib, transform, region, mask);
+	status = (*s->paintWindow) (w, &sAttrib, transform, region, mask);
 	WRAP (ss, s, paintWindow, shiftPaintWindow);
     }
 
     return status;
 }
 
-static inline float 
-shiftLinearInterpolation(float valX,
-			float minX, float maxX, 
-			float minY, float maxY)
-{
-    double factor = (maxY - minY) / (maxX - minX);
-    return (minY + (factor * (valX - minX)));
-}
-
 static int
 compareWindows (const void *elem1,
 		const void *elem2)
 {
     CompWindow *w1 = *((CompWindow **) elem1);
     CompWindow *w2 = *((CompWindow **) elem2);
+    CompWindow *w  = w1;
 
+    if (w1 == w2)
+	return 0;
+
+    if (!w1->shaded && w1->attrib.map_state != IsViewable &&
+        (w2->shaded || w2->attrib.map_state == IsViewable))
+    {
+	return 1;
+    }
+
+    if (!w2->shaded && w2->attrib.map_state != IsViewable &&
+        (w1->shaded || w1->attrib.map_state == IsViewable))
+    {
+	return -1;
+    }
+
+    while (w)
+    {
+	if (w == w2)
+	    return 1;
+	w = w->next;
+    }
+    return -1;
+
+    /*
     if (w1->mapNum && !w2->mapNum)
 	return -1;
 
@@ -712,6 +781,7 @@ compareWindows (const void *elem1,
 	return 1;
 
     return w2->activeNum - w1->activeNum;
+    */
 }
 
 static int 
@@ -740,10 +810,21 @@ layoutThumbsCover (CompScreen *s)
     float distance;
     int i;
 
-    int ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
-    int ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
-    int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
-    int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    int ox1, ox2, oy1, oy2;
+
+    if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
+    {
+	ox1 = oy1 = 0;
+	ox2 = s->width;
+	oy2 = s->height;
+    }
+    else
+    {
+        ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
+        ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
+        oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
+        oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    }
     
     /* the center of the ellipse is in the middle 
        of the used output device */
@@ -758,8 +839,6 @@ layoutThumbsCover (CompScreen *s)
 	w = ss->windows[index];
 	SHIFT_WINDOW (w);
 
- 	sw->slots[0].active = TRUE;
-
 	ww = w->width  + w->input.left + w->input.right;
 	wh = w->height + w->input.top  + w->input.bottom;
 
@@ -842,6 +921,19 @@ layoutThumbsCover (CompScreen *s)
 		ss->drawSlots[index * 2 + i].distance = fabs(distance);
 		
 	}
+
+	if (ss->drawSlots[index * 2].distance >
+	    ss->drawSlots[index * 2 + 1].distance)
+	{
+	    ss->drawSlots[index * 2].slot->primary     = FALSE;
+	    ss->drawSlots[index * 2 + 1].slot->primary = TRUE;
+	}
+	else
+	{
+	    ss->drawSlots[index * 2].slot->primary     = TRUE;
+	    ss->drawSlots[index * 2 + 1].slot->primary = FALSE;
+	}
+
     }
 
     ss->nSlots = ss->nWindows * 2;
@@ -865,10 +957,21 @@ layoutThumbsFlip (CompScreen *s)
     float angle;
     int slotNum;
 
-    int ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
-    int ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
-    int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
-    int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    int ox1, ox2, oy1, oy2;
+
+    if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
+    {
+	ox1 = oy1 = 0;
+	ox2 = s->width;
+	oy2 = s->height;
+    }
+    else
+    {
+        ox1 = s->outputDev[ss->usedOutput].region.extents.x1;
+        ox2 = s->outputDev[ss->usedOutput].region.extents.x2;
+        oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
+        oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+    }
     
     /* the center of the ellipse is in the middle 
        of the used output device */
@@ -885,8 +988,6 @@ layoutThumbsFlip (CompScreen *s)
 	w = ss->windows[index];
 	SHIFT_WINDOW (w);
 
- 	sw->slots[0].active = TRUE;
-
 	ww = w->width  + w->input.left + w->input.right;
 	wh = w->height + w->input.top  + w->input.bottom;
 
@@ -925,6 +1026,12 @@ layoutThumbsFlip (CompScreen *s)
 			sw->slots[i].opacity = 1.0;
 		}
 
+		if (distance > 0.0 && w->id != ss->selectedWindow)
+		    sw->slots[i].primary = FALSE;
+		else
+		    sw->slots[i].primary = TRUE;
+
+
 		sw->slots[i].scale   = MIN (xScale, yScale);
 		
 		sw->slots[i].y = centerY + (maxThumbHeight / 2.0) -
@@ -1039,7 +1146,6 @@ static Bool
 shiftCreateWindowList (CompScreen *s)
 {
     CompWindow *w;
-    int i;
     SHIFT_SCREEN (s);
 
     ss->nWindows = 0;
@@ -1052,9 +1158,6 @@ shiftCreateWindowList (CompScreen *s)
 
 	    shiftAddWindowToList (s, w);
 	    sw->active = TRUE;
-
-	    for (i = 0; i < 2; i++)
-	    	sw->slots[i].adjust = TRUE;
 	}
     }
 
@@ -1176,41 +1279,6 @@ static int adjustShiftMovement (CompScreen *s, float chunk)
     return TRUE;
 }
 
-static Bool
-adjustShiftVelocity (CompWindow *w, ShiftSlot *slot)
-{
-    float dp, adjust, amount;
-    float opacity;
-
-    SHIFT_WINDOW (w);
-
-    if (sw->active)
-	opacity = slot->opacity;
-    else
-	opacity = 0.0;
-
-    dp = opacity - slot->topacity;
-    adjust = dp * 0.1f;
-    amount = fabs (dp) * 7.0f;
-    if (amount < 0.01f)
-	amount = 0.01f;
-    else if (amount > 0.15f)
-	amount = 0.15f;
-
-    slot->opacityVelocity = (amount * slot->opacityVelocity + adjust) /
-	(amount + 1.0f);
-
-
-    if (fabs (dp) < 0.01f && fabs (slot->opacityVelocity) < 0.02f)
-    {
-	slot->topacity = opacity;
-
-	return FALSE;
-    }
-
-    return TRUE;
-}
-
 static Bool
 adjustShiftWindowAttribs (CompWindow *w, float chunk)
 {
@@ -1220,7 +1288,10 @@ adjustShiftWindowAttribs (CompWindow *w, float chunk)
     SHIFT_WINDOW (w);
     SHIFT_SCREEN (w->screen);
 
-    if (sw->active && ss->state != ShiftStateIn && ss->state != ShiftStateNone)
+    if ((sw->active && ss->state != ShiftStateIn &&
+	ss->state != ShiftStateNone) ||
+	(shiftGetHideAll(w->screen) && !(w->type & CompWindowTypeDesktopMask) &&
+	(ss->state == ShiftStateOut || ss->state == ShiftStateSwitching)))
 	opacity = 0.0;
     else
 	opacity = 1.0;
@@ -1269,37 +1340,37 @@ adjustShiftWindowAttribs (CompWindow *w, float chunk)
 }
 
 static Bool
-adjustShiftReflectionAttribs (CompScreen *s, float chunk)
+adjustShiftAnimationAttribs (CompScreen *s, float chunk)
 {
     float dr, adjust, amount;
-    float reflect;
+    float anim;
 
     SHIFT_SCREEN (s);
 
     if (ss->state != ShiftStateIn && ss->state != ShiftStateNone)
-	reflect = 1.0;
+	anim = 1.0;
     else
-	reflect = 0.0;
+	anim = 0.0;
 
-    dr = reflect - ss->reflect;
+    dr = anim - ss->anim;
     adjust = dr * 0.1f;
     amount = fabs (dr) * 7.0f;
-    if (amount < 0.01f)
-	amount = 0.01f;
+    if (amount < 0.002f)
+	amount = 0.002f;
     else if (amount > 0.15f)
 	amount = 0.15f;
 
-    ss->reflectVelocity = (amount * ss->reflectVelocity + adjust) /
+    ss->animVelocity = (amount * ss->animVelocity + adjust) /
 	(amount + 1.0f);
 
-    if (fabs (dr) < 0.01f && fabs (ss->reflectVelocity) < 0.02f)
+    if (fabs (dr) < 0.002f && fabs (ss->animVelocity) < 0.004f)
     {
 
-	ss->reflect = reflect;
+	ss->anim = anim;
 	return FALSE;
     }
 
-    ss->reflect += ss->reflectVelocity * chunk;
+    ss->anim += ss->animVelocity * chunk;
     return TRUE;
 }
 
@@ -1318,13 +1389,16 @@ shiftPaintOutput (CompScreen		  *s,
     if (ss->state != ShiftStateNone)
 	mask |= PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK;
 
+    ss->paintingAbove = FALSE;
+
     ss->output = output;
 
     UNWRAP (ss, s, paintOutput);
     status = (*s->paintOutput) (s, sAttrib, transform, region, output, mask);
     WRAP (ss, s, paintOutput, shiftPaintOutput);
 
-    if (ss->state != ShiftStateNone && output->id == ss->usedOutput)
+    if (ss->state != ShiftStateNone &&
+	(output->id == ss->usedOutput || output->id == ~0))
     {
 	int           i;
 	
@@ -1332,6 +1406,13 @@ shiftPaintOutput (CompScreen		  *s,
 	CompTransform sTransform = *transform;
 	int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
 	int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+
+	if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
+	{
+	    oy1 = 0;
+	    oy2 = s->height;
+	}
+	
 	int maxThumbHeight = (oy2 - oy1) * shiftGetSize(s) / 100;
 
 	int oldFilter = s->display->textureFilter;
@@ -1348,9 +1429,6 @@ shiftPaintOutput (CompScreen		  *s,
 
 	    unsigned short color[4];
 
-	    int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
-	    int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
-
 	    int maxThumbHeight = (oy2 - oy1) * shiftGetSize(s) / 100;
 
 	    matrixTranslate (&rTransform, 0.0, (oy2 - oy1) + maxThumbHeight,
@@ -1366,12 +1444,13 @@ shiftPaintOutput (CompScreen		  *s,
 		s->display->textureFilter = GL_LINEAR_MIPMAP_LINEAR;
 
 
-	    if (ss->reflect == 1.0)
+	    if (ss->anim == 1.0)
 	    {
 		glClipPlane (GL_CLIP_PLANE0, clip);
 		glEnable (GL_CLIP_PLANE0);
 	    }
 
+	    ss->reflectActive = TRUE;
 	    ss->reflectBrightness = shiftGetIntensity(s);
 	    for (i = 0; i < ss->nSlots; i++)
 	    {
@@ -1399,7 +1478,7 @@ shiftPaintOutput (CompScreen		  *s,
 	    glVertex2f (-0.5, 0.0);
 	    glColor4f (0.0, 0.0, 0.0,
 		       MIN (1.0, 1.0 - shiftGetIntensity (s)) * 2.0 *
-		       ss->reflect);
+		       ss->anim);
 	    glVertex2f (-0.5, -0.5);
 	    glVertex2f (0.5, -0.5);
 	    glEnd();
@@ -1410,14 +1489,14 @@ shiftPaintOutput (CompScreen		  *s,
 		color[0] = shiftGetGroundColor1 (s)[0];
 		color[1] = shiftGetGroundColor1 (s)[1];
 		color[2] = shiftGetGroundColor1 (s)[2];
-		color[3] = (float)shiftGetGroundColor1 (s)[3] * ss->reflect;
+		color[3] = (float)shiftGetGroundColor1 (s)[3] * ss->anim;
 		glColor4usv (color);
 		glVertex2f (-0.5, -0.5);
 		glVertex2f (0.5, -0.5);
 		color[0] = shiftGetGroundColor2 (s)[0];
 		color[1] = shiftGetGroundColor2 (s)[1];
 		color[2] = shiftGetGroundColor2 (s)[2];
-		color[3] = (float)shiftGetGroundColor2 (s)[3] * ss->reflect;
+		color[3] = (float)shiftGetGroundColor2 (s)[3] * ss->anim;
 		glColor4usv (color);
 		glVertex2f (0.5, -0.5 + shiftGetGroundSize (s));
 		glVertex2f (-0.5, -0.5 + shiftGetGroundSize (s));
@@ -1433,13 +1512,14 @@ shiftPaintOutput (CompScreen		  *s,
 	glPushMatrix ();
 	glLoadMatrixf (sTransform.m);
 
-	if (shiftGetReflection (s) && ss->reflect == 1.0)
+	if (shiftGetReflection (s) && ss->anim == 1.0)
 	{
 	    glClipPlane (GL_CLIP_PLANE0, clip);
 	    glEnable (GL_CLIP_PLANE0);
 	}
 
 	ss->reflectBrightness = 1.0;
+	ss->reflectActive     = FALSE;
 	
 	for (i = 0; i < ss->nSlots; i++)
 	{
@@ -1460,13 +1540,65 @@ shiftPaintOutput (CompScreen		  *s,
 
 	if (ss->textPixmap && (ss->state != ShiftStateIn))
 	    shiftDrawWindowTitle (s);
+
+	if (ss->state == ShiftStateIn || ss->state == ShiftStateOut)
+	{
+	    Bool found;
+	    ss->paintingAbove = TRUE;
+
+	    w = findWindowAtScreen (s, ss->selectedWindow);
+	    
+	    for (; w; w = w->next)
+	    {
+		if (w->destroyed)
+		    continue;
+
+		if (!w->shaded)
+		{
+		    if (w->attrib.map_state != IsViewable || !w->damaged)
+			continue;
+		}
+		found = FALSE;
+		for (i = 0; i < ss->nWindows; i++)
+		    if (ss->windows[i] == w)
+			found = TRUE;
+		if (found)
+		    continue;
+		(*s->paintWindow) (w, &w->paint, &sTransform,
+				   &infiniteRegion, 0);
+	    }
+
+	    ss->paintingAbove = FALSE;
+	}	
 	
 	glPopMatrix ();
+
+
     }
 
     return status;
 }
 
+static void
+shiftPaintScreen (CompScreen   *s,
+		  CompOutput   *outputs,
+		  int          numOutputs,
+		  unsigned int mask)
+{
+    SHIFT_SCREEN (s);
+
+    if (ss->state != ShiftStateNone && numOutputs > 0 &&
+        shiftGetMultioutputMode (s) != MultioutputModeDisabled)
+    {
+	outputs = &s->fullscreenOutput;
+	numOutputs = 1;
+    }
+
+    UNWRAP (ss, s, paintScreen);
+    (*s->paintScreen) (s, outputs, numOutputs, mask);
+    WRAP (ss, s, paintScreen, shiftPaintScreen);
+}
+
 static void
 shiftPreparePaintScreen (CompScreen *s,
 			int	    msSinceLastPaint)
@@ -1505,7 +1637,7 @@ shiftPreparePaintScreen (CompScreen *s,
 
 	while (steps--)
 	{
-	    ss->moreAdjust = adjustShiftReflectionAttribs (s, chunk);
+	    ss->moreAdjust = adjustShiftAnimationAttribs (s, chunk);
 
 	    for (w = s->windows; w; w = w->next)
 	    {
@@ -1515,22 +1647,10 @@ shiftPreparePaintScreen (CompScreen *s,
 		for (i = 0; i < 2; i++)
 		{
 		    ShiftSlot *slot = &sw->slots[i];
-		    if (slot->adjust)
-		    {
-			slot->adjust = adjustShiftVelocity (w, slot);
-			
-			ss->moreAdjust |= slot->adjust;
-			
-			slot->topacity += slot->opacityVelocity * chunk;
-		    }
-		    else
-			slot->topacity = slot->opacity;
-
 		    slot->tx = slot->x - w->attrib.x -
 			(w->attrib.width * slot->scale) / 2;
 		    slot->ty = slot->y - w->attrib.y -
 			(w->attrib.height * slot->scale) / 2;
-		    slot->topacity = MIN (1.0, MAX (0.0, slot->topacity));
 		}
 	    }
 
@@ -1548,6 +1668,7 @@ static void
 shiftDonePaintScreen (CompScreen *s)
 {
     SHIFT_SCREEN (s);
+    CompWindow *w;
 
     if (ss->state != ShiftStateNone)
     {
@@ -1566,6 +1687,12 @@ shiftDonePaintScreen (CompScreen *s)
 	    {
 		ss->state = ShiftStateNone;
 		shiftActivateEvent(s, FALSE);
+		for (w = s->windows; w; w = w->next)
+		{
+		    SHIFT_WINDOW (w);
+		    sw->active = FALSE;
+		}
+		damageScreen (s);
 	    }
 	    else if (ss->state == ShiftStateOut)
 		ss->state = ShiftStateSwitching;
@@ -1577,6 +1704,22 @@ shiftDonePaintScreen (CompScreen *s)
     WRAP (ss, s, donePaintScreen, shiftDonePaintScreen);
 }
 
+static Bool
+canStackRelativeTo (CompWindow *w)
+{
+    if (w->attrib.override_redirect)
+        return FALSE;
+
+    if (!w->shaded && !w->pendingMaps)
+    {
+        if (w->attrib.map_state != IsViewable || w->mapNum == 0)
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
+
 static void
 shiftTerm (CompScreen *s, Bool cancel)
 {
@@ -1592,19 +1735,20 @@ shiftTerm (CompScreen *s, Bool cancel)
     {
 	CompWindow *w;
 
-	for (w = s->windows; w; w = w->next)
+	CompWindow *pw = NULL;
+	int i;
+	
+	for (i = 0; i < ss->nSlots; i++)
 	{
-	    SHIFT_WINDOW (w);
-
-	    sw->slots[0].active = FALSE;
-	    sw->slots[1].active = FALSE;
-	    if (sw->active)
+	    w = ss->drawSlots[i].w;
+	    if (ss->drawSlots[i].slot->primary && canStackRelativeTo (w))
 	    {
-		sw->slots[0].adjust = TRUE;
-		sw->slots[1].adjust = TRUE;
+		if (pw)
+		    restackWindowAbove (w,pw);
+		pw = w;
 	    }
-	    sw->active = FALSE;
 	}
+
 	ss->moreAdjust = TRUE;
 	ss->state = ShiftStateIn;
 	damageScreen (s);
@@ -1615,6 +1759,7 @@ shiftTerm (CompScreen *s, Bool cancel)
 	    if (w)
 		sendWindowActivationRequest (s, w->id);
 	}
+	
     }
 }
 
@@ -1766,7 +1911,7 @@ shiftDoSwitch (CompDisplay     *d,
 	if (ret)
 	{
     	    switchToWindow (s, nextWindow);
-	    if (initial)
+	    if (initial && FALSE)
 	    {
 		ss->mvTarget += ss->mvAdjust;
 		ss->mvAdjust  = 0.0;
@@ -2219,8 +2364,6 @@ shiftDamageWindowRect (CompWindow *w,
 		SHIFT_WINDOW (w);
 
     		sw->active = TRUE;
-		sw->slots[0].adjust = TRUE;
-		sw->slots[1].adjust = TRUE;
 		ss->moreAdjust = TRUE;
 		ss->state = ShiftStateOut;
 		damageScreen (w->screen);
@@ -2348,8 +2491,8 @@ shiftInitScreen (CompPlugin *p,
 
     ss->textPixmap = None;
 
-    ss->reflect         = 0.0;
-    ss->reflectVelocity = 0.0;
+    ss->anim         = 0.0;
+    ss->animVelocity = 0.0;
 
     ss->buttonPressed = FALSE;
 
@@ -2357,6 +2500,7 @@ shiftInitScreen (CompPlugin *p,
 
     WRAP (ss, s, preparePaintScreen, shiftPreparePaintScreen);
     WRAP (ss, s, donePaintScreen, shiftDonePaintScreen);
+    WRAP (ss, s, paintScreen, shiftPaintScreen);
     WRAP (ss, s, paintOutput, shiftPaintOutput);
     WRAP (ss, s, paintWindow, shiftPaintWindow);
     WRAP (ss, s, damageWindowRect, shiftDamageWindowRect);
@@ -2378,6 +2522,7 @@ shiftFiniScreen (CompPlugin *p,
 
     UNWRAP (ss, s, preparePaintScreen);
     UNWRAP (ss, s, donePaintScreen);
+    UNWRAP (ss, s, paintScreen);
     UNWRAP (ss, s, paintOutput);
     UNWRAP (ss, s, paintWindow);
     UNWRAP (ss, s, damageWindowRect);
