@@ -101,6 +101,8 @@ typedef struct _WallScreen
     int boxTimeout;
     int boxOutputDevice;
 
+    int timer;
+
     Window moveWindow;
 
     Bool              miniScreen;
@@ -340,7 +342,6 @@ static void
 wallDrawArrow (CompScreen *s)
 {
     cairo_t *cr;
-    float   width, height;
     float   outline = 2.0f;
     float   r, g, b, a;
 
@@ -349,9 +350,6 @@ wallDrawArrow (CompScreen *s)
     cr = ws->arrowContext.cr;
     wallClearCairoLayer (cr);
 
-    width  = (float) ws->arrowContext.width - outline;
-    height = (float) ws->arrowContext.height - outline;
-
     cairo_translate (cr, outline / 2.0f, outline / 2.0f);
 
     /* apply the pattern for thumb background */
@@ -474,6 +472,30 @@ wallReleaseMoveWindow (CompScreen *s)
     ws->moveWindow = 0;
 }
 
+static void
+wallComputeTranslation (CompScreen *s,
+			float      *x,
+			float      *y)
+{
+    float dx, dy, elapsed;
+
+    WALL_SCREEN (s);
+
+    elapsed = 1 - (ws->timer / (wallGetSlideDuration (s->display) * 1000));
+
+    if (elapsed < 0.0)
+	elapsed = 0.0;
+    if (elapsed > 1.0)
+	elapsed = 1.0;
+
+    /* Use temporary variables to you can pass in &ps->cur_x */
+    dx = (ws->gotoX - ws->curPosX) * elapsed + ws->curPosX;
+    dy = (ws->gotoY - ws->curPosY) * elapsed + ws->curPosY;
+
+    *x = dx;
+    *y = dy;
+}
+
 static Bool
 wallMoveViewport (CompScreen *s,
 		  int        x,
@@ -535,6 +557,8 @@ wallMoveViewport (CompScreen *s,
 	    ws->boxTimeout = 0;
 	    ws->moving = FALSE;
 	}
+
+	ws->timer = wallGetSlideDuration (s->display) * 1000;
     }
 
     damageScreen (s);
@@ -641,28 +665,39 @@ wallPrev (CompDisplay     *d,
     return TRUE;
 }
 
+static void
+wallCheckAmount (CompScreen *s,
+		 int        dx,
+		 int        dy,
+		 int        *amountX,
+		 int        *amountY)
+{
+    *amountX = -dx;
+    *amountY = -dy;
+
+    if (wallGetAllowWraparound (s->display))
+    {
+	if ((s->x + dx) < 0)
+	    *amountX = -(s->hsize + dx);
+	else if ((s->x + dx) >= s->hsize)
+	    *amountX = s->hsize - dx;
+
+	if ((s->y + dy) < 0)
+	    *amountY = -(s->vsize + dy);
+	else if ((s->y + dy) >= s->vsize)
+	    *amountY = s->vsize - dy;
+    }
+}
+
 static Bool
 wallInitiate (CompScreen *s,
 	      int        dx,
 	      int        dy,
 	      Window     win)
 {
-    int amountX = -dx;
-    int amountY = -dy;
-
-    if (wallGetAllowWraparound (s->display))
-    {
-	if ((s->x + dx) < 0)
-	    amountX = -(s->hsize + dx);
-	else if ((s->x + dx) >= s->hsize)
-	    amountX = s->hsize - dx;
-
-	if ((s->y + dy) < 0)
-	    amountY = -(s->vsize + dy);
-	else if ((s->y + dy) >= s->vsize)
-	    amountY = s->vsize - dy;
-    }
+    int amountX, amountY;
 
+    wallCheckAmount (s, dx, dy, &amountX, &amountY);
     wallMoveViewport (s, amountX, amountY, win);
 
     return TRUE;
@@ -674,6 +709,7 @@ wallInitiateFlip (CompScreen *s,
 		  Bool       dnd)
 {
     int dx, dy;
+    int amountX, amountY;
 
     if (otherScreenGrabExist (s, "wall", "move", "group-drag", 0))
 	return FALSE;
@@ -704,33 +740,34 @@ wallInitiateFlip (CompScreen *s,
     switch (direction)
     {
     case Left:
-	dx = 1; dy = 0;
-	break;
-    case Right:
 	dx = -1; dy = 0;
 	break;
+    case Right:
+	dx = 1; dy = 0;
+	break;
     case Up:
-	dx = 0; dy = 1;
+	dx = 0; dy = -1;
 	break;
     case Down:
-	dx = 0; dy = -1;
+	dx = 0; dy = 1;
 	break;
     default:
 	dx = 0; dy = 0;
 	break;
     }
 
-    if (wallMoveViewport (s, dx, dy, None))
+    wallCheckAmount (s, dx, dy, &amountX, &amountY);
+    if (wallMoveViewport (s, amountX, amountY, None))
     {
 	int offsetX, offsetY;
 	int warpX, warpY;
 
-	if (dx > 0)
+	if (dx < 0)
 	{
 	    offsetX = s->width - 10;
 	    warpX = pointerX + s->width;
 	}
-	else if (dx < 0)
+	else if (dx > 0)
 	{
 	    offsetX = 1- s->width;
 	    warpX = pointerX - s->width;
@@ -741,12 +778,12 @@ wallInitiateFlip (CompScreen *s,
 	    warpX = lastPointerX;
 	}
 
-	if (dy > 0)
+	if (dy < 0)
 	{
 	    offsetY = s->height - 10;
 	    warpY = pointerY + s->height;
 	}
-	else if (dy < 0)
+	else if (dy > 0)
 	{
 	    offsetY = 1- s->height;
 	    warpY = pointerY - s->height;
@@ -1289,20 +1326,12 @@ wallPreparePaintScreen (CompScreen *s,
     if (!ws->moving && ws->boxTimeout)
 	ws->boxTimeout -= msSinceLastPaint;
 
+    if (ws->timer)
+	ws->timer -= msSinceLastPaint;
+
     if (ws->moving)
     {
-	float dx, dy, mv;
-
-	mv = (float)msSinceLastPaint /
-	     (wallGetSlideDuration (s->display) * 1000.0f);
-
-	dx = ws->gotoX - ws->curPosX;
-	dy = ws->gotoY - ws->curPosY;
-
-	ws->curPosX = (dx > 0) ? MIN (ws->curPosX + mv, ws->gotoX) :
-	                         MAX (ws->curPosX - mv, ws->gotoX);
-	ws->curPosY = (dy > 0) ? MIN (ws->curPosY + mv, ws->gotoY) :
-	                         MAX(ws->curPosY - mv, ws->gotoY);
+	wallComputeTranslation (s, &ws->curPosX, &ws->curPosY);
 
 	if (ws->moveWindow)
 	{
@@ -1311,12 +1340,11 @@ wallPreparePaintScreen (CompScreen *s,
 	    w = findWindowAtScreen (s, ws->moveWindow);
 	    if (w)
     	    {
+		float dx, dy;
+
 		dx = ws->gotoX - ws->curPosX;
 		dy = ws->gotoY - ws->curPosY;
 
-		moveWindow (w,
-			    ws->moveWindowX - w->attrib.x,
-			    ws->moveWindowY - w->attrib.y, TRUE, TRUE);
 		moveWindowToViewportPosition (w,
 					      ws->moveWindowX - s->width * dx,
 					      ws->moveWindowY - s->height * dy,
@@ -1328,6 +1356,7 @@ wallPreparePaintScreen (CompScreen *s,
     if (ws->moving && ws->curPosX == ws->gotoX && ws->curPosY == ws->gotoY)
     {
 	ws->moving = FALSE;
+	ws->timer  = 0;
 
 	if (ws->moveWindow)
 	    wallReleaseMoveWindow (s);
